/*
   Licensed Materials - Property of IBM
   5724-M22, 5724-M23, 5724-M24, 5724-L01, 5724-R31
   Copyright IBM Corporation 2007, 2008. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bspace.widgets.samples.ModelSample");

//This widget displays a list of installed monitor models
dojo.declare("com.ibm.bspace.widgets.samples.ModelSample",[],{
	// The REST URI used to retrieve models
	MODEL_REST_URI: "bpm/monitor/models", 

    onLoad: function () {
		try {
			console.debug("Entering onLoad."); 
			// Mixin the BusinessSpace utility classes to this class and load the Business Space general helper
    	    // These provide access to libraries and functions that WebSphere Business Space provides
        	dojo.mixin(this, new com.ibm.bspace.common.util.widget.BSpaceCommonUtilityLoader());
			this.require("com.ibm.bspace.common.util.widget.BSpaceGeneralHelper");
			this.printDebug("ModelSample; method onLoad entry; iWidgetId="+this.iContext.widgetId);
			//Require the Business Space Registry helper which provides access
			//to registry elements, such as widget and service endpoints.
			this.require("com.ibm.bspace.common.util.widget.BSpaceWidgetRegistry"); 
			dojo.addOnLoad(this, "_onLoad");
			console.debug("Exiting onLoad."); 
			this.printDebug("ModelSample; method onLoad exit; iWidgetId="+this.iContext.widgetId);
		} catch (ex) {
			//call the mixed-in exception handler to catch/log any exceptions that occur in this function 
			this.handleException("ModelSample", "onLoad()", ex.message, ex);
		}
    }, 
	
	onReload: function () {
    	try {
    		dojo.addOnLoad(this, "_onLoad");
    	} catch (ex) {
			//call the mixed-in exception handler to catch/log any exceptions that occur in this function 
			this.handleException("ModelSample", "onReload()", ex.message, ex);
		}
    }, 
    
    onview: function(){
 		try {
 			this.setGridDimension();
 			this.showFilterPattern();
 			this.findModels(this.listModels); //find models then list them in widget
 		} catch (ex) {
			//call the mixed-in exception handler to catch/log any exceptions that occur in this function 
			this.handleException("ModelSample", "onview()", ex.message, ex);
		}
    },
    
    // Clean up elements when unloaded. 
 	onUnload: function(){
    	try {
    		this.cleanElements();
    	} catch (ex) {
			//call the mixed-in exception handler to catch/log any exceptions that occur in this function 
			this.handleException("ModelSample", "onUnload()", ex.message, ex);
		}
 	},
	
 	//
	//	Put any extra loading info that needs to be deferred until the page is fully available in the _onLoad
	//
	_onLoad: function () {
    	dojo.require("dojox.grid.Grid");
    	dojo.require("dojox.grid._data.model");
    	dojo.require("dojo.data.ItemFileReadStore");
    	dojo.require("dojox.lang.functional"); 
    	if(dojo.version.major >= 1 && dojo.version.minor >= 1){
    		dojo.require("dojox.lang.functional.fold");
    	}
    	//register the samples modules path and set up the localization
    	dojo.registerModulePath("com.ibm.bspace.widgets.samples", this.iContext.io.rewriteURI("../"));
    	dojo.requireLocalization("com.ibm.bspace.widgets.samples", "widgetResources");
        this._messages = dojo.i18n.getLocalization("com.ibm.bspace.widgets.samples", "widgetResources");
    }, 
    
    // Clean all the created elements to prevent memory leaks
    cleanElements: function(){
    	dojo.forEach(this._registeredElements, function(w){
    		if(w["destroy"]){
    			try{
    				w.destroy(); 
    			}catch(error){
    				console.debug("error when cleaning elements: "+error); 
    			}
    			w = null; 
    		}
    	});
    },
    
    // This function is an example of how to get access to attributes that have been defined in the iWidget
    // definition file and how to modify a DOM node that is defined in the view section of the widget definition file.
    // The function displays the pattern retrieved from the iWidget attribute bspace_filterPattern. 
    showFilterPattern: function(){
    	//first get the node id and prefix it with the widget id in the iContext.  This will replace the _IWID_
    	//pattern in the widget's XML definition file
    	var nodeId = "_" + this.iContext.widgetId + "_" +"bspace_filterPattern";
    	//now get the node by Id, retrieve the attribute from the iContext, and set the node's value
    	var node = this.iContext.getElementById(nodeId)
    	//allowedModels is a regularExpression filter in the widget definition file
    	var allowedModels = this.iContext.getiWidgetAttributes().getItemValue("allowedModels");
    	node.innerHTML = "<h4>"+this._messages.filterPatternLabel + " /" + allowedModels + "/</h4>";
    }, 
    
    // Sets the height and width of the grid container. A grid container must have dimensions before its grid is parsed. 
    // Otherwise, the grid will not show. 
    setGridDimension: function(){
    	var height = "300px";
    	var width = "500px"; 
    	dojo.forEach(this.iContext.getElementByClass("bspaceModelSample_gridContainer"), function(node){
    		dojo.style(node, "height", height); 
    		dojo.style(node, "width", width);
    	}); 
    }, 
    
    // Track created elements, useful for destroying them when they are out of scope
    registerElement: function(element){
    	this._registeredElements = this._registeredElements || []; 
    	this._registeredElements.push(element);
    }, 
    
    // Find all the models on the Monitor server, and then apply the given action.
    // the "getServiceEndpoint" is mixed in from the Registry helper and provides access to the
    // Business Space Registry
    findModels: function(action){
    	 dojo.xhrGet({
            url: this.getServiceEndpoint("serviceUrlRoot")+this.MODEL_REST_URI,
            	//serviceUrlRoot can be found in the widget registry file 
            load: dojo.hitch(this, action), 
            error: function( data, ioArgs ) {
                console.error("Failed to get list of models: ", data);
            },
            sync: false
        });
    }, 
    
    // List models in the response in a grid. The response is assumed to be JSON-formatted text. 
    listModels: function(response, ioArgs){
    	//allowedModels is a regularExpression filter in the widget definition file
    	var allowedModels = this.iContext.getiWidgetAttributes().getItemValue("allowedModels");
    	var models = this.selectAllowedModels(dojo.fromJson(response), allowedModels); 
    	var nodeId = "_" + this.iContext.widgetId + "_" +"bspaceModelSample_modelList";
    	this.createModelGrid(this.convertData(models), this.iContext.getElementById(nodeId)); 
    }, 
    
	// Selects the models whose model IDs match the given pattern. 
    selectAllowedModels: function(/*Array*/ models, modelPatterns){
    	if(!modelPatterns || dojo.trim(modelPatterns).length == 0){
    		// Nothing to be filtered out
    		return models; 
    	}
    	modelPatterns = new RegExp(modelPatterns, "i");
    	return dojo.filter(models, function(model){
    		return modelPatterns.test(model["Model ID"]); 
    	});
    }, 
    
    // Helper function to convert one data structure into another 
    // convert: [{"Model ID":"m","Versions":[{"Version":v,"Model Display Name":"n"}]}] 
    // to {items: [{modelId: "m", version:v, displayName: n}]}
    convertData: function(models){
    	var items = dojox.lang.functional.foldl(models, function(result, model){
			var versions = dojo.map(model["Versions"], function(version){
    			return {
    				modelId: model["Model ID"], 
    				version: version["Version"], 
    				displayName: version["Model Display Name"]
    			};
    		}); 
    		return result.concat(versions); 
    	}, []); 
    	return {items: items}; 
    }, 
    
    // Creates a grid that shows the given model data. The grid is contained in the given container node. 
 	createModelGrid: function(modelData, containerNode){
 		// a grid view is a group of columns
		var view1 = {
			cells: [[
				{name: this._messages.gridColumnModelID, field: "modelId"}, 
				{name: this._messages.gridColumnVersion,  field: "version"}, 
				{name: this._messages.gridColumnModelDisplayName, width:"25em", field: "displayName"}
			  ]
			]
		};
		// a grid layout is an array of views. 
		var layout = [view1]; 
		var jsonStore = new dojo.data.ItemFileReadStore({data: modelData}); 
		jsonStore.fetch({query: {"modelId":'*'}}); 
		if(dijit.byId(this.iContext.widgetId + "_grid")){
			dijit.byId(this.iContext.widgetId + "_grid").destroy(); 
		}
		var grid = new dojox.Grid({
			id: this.iContext.widgetId + "_grid",
			keepRows: 30, 
			clientSort: true, 
			model: new dojox.grid.data.DojoData(null, jsonStore, {rowsPerPage: 10, query: {"modelId":'*'}}), 
			structure: layout}, containerNode); 
		grid.render();
		this.registerElement(grid);
 	}
});



